﻿Type=Class
Version=2.31
ModulesStructureVersion=1
B4i=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Type Beacon (uniqueid As String, BeaconType As Int, TIME As Long, SpecificData As Object, Distance As Double)
	Type iBeacon (Uuid As String, Minor As Int, Major As Int)
	Type EddystoneUID (Namespace As String, Instance As String)
	Type EddystoneURL (url As String) 
	Public const TypeiBeacon = 1, TypeEddystoneUID = 2, TypeEddystoneURL = 3 As Int
	Private bc As ByteConverter
	Private urlSchemes As List = Array("http://www.", "https://www.", "http://", "https://")
	Private urlExpansion As List = Array(".com/", ".org/", ".edu/", ".net/", ".info/", ".biz/", ".gov/", _
		".com", ".org", ".edu", ".net", ".info", ".biz", ".gov")
End Sub

Public Sub Initialize
End Sub

Public Sub Parse(AdvertisingData As Map, RSSI As Int) As Beacon
'	For Each key As Int In AdvertisingData.Keys
'		Log($"${key}= ${bc.HexFromBytes(AdvertisingData.Get(key))}"$)
'	Next
	
	Dim beacon1 As Beacon
	beacon1.Initialize
	Dim tx As Byte
#if B4A
	Dim key As Int = -1 'type is important. Must be Int.
	If AdvertisingData.ContainsKey(key) Then
		Dim b() As Byte = AdvertisingData.Get(key)
		If CompareBytes(b, Array As Byte(0x4c, 0, 0x02, 0x15), 0) Then
			'iBeacon
			beacon1.BeaconType = TypeiBeacon
			Dim raf As RandomAccessFile
			Dim ib As iBeacon
			raf.Initialize3(b, False)
			Dim hex As String = bc.HexFromBytes(b)
			ib.uuid = hex.SubString2(8, 40) 'bytes 4 - 19
			beacon1.uniqueid = hex.SubString2(8, 48) 'this also includes the major and minor parts
			ib.major = raf.ReadShort(20)
			ib.minor = raf.ReadShort(22)
			tx = raf.ReadSignedByte(24)
			beacon1.SpecificData = ib
		End If
	End If
	
	If beacon1.BeaconType = 0 Then
		key = 22
		If AdvertisingData.ContainsKey(key) Then
			Dim b() As Byte = AdvertisingData.Get(key)
			If CompareBytes(b, Array As Byte(0xAA, 0xFE), 0) Then
				'Eddystone
				tx = ParseEddystone(beacon1, b, 0)
			End If
		End If
	End If
#else if B4I
	If AdvertisingData.ContainsKey("kCBAdvDataServiceData") Then
		Dim b() As Byte
		Dim no As NativeObject = Me
		no.RunMethod("fillData:::", Array("FEAA", b, AdvertisingData.Get("kCBAdvDataServiceData")))
		tx = ParseEddystone(beacon1, b, 2)
	End If
#if OBJC
@import CoreBluetooth;
- (void) fillData:(NSString*)key :(B4IArray*)array :(NSDictionary*)dict {
	CBUUID* uuid = [CBUUID UUIDWithString:key];
	NSData* data = dict[uuid];
	if (data != nil) {
		array.bytesData = data;
	}
}
#end if
#End If
	
	
	If beacon1.BeaconType <> 0 Then
		beacon1.distance = CalculateDistance(tx, RSSI)
		beacon1.time = DateTime.Now
		Return beacon1
	Else
		Return Null
	End If
End Sub

Private Sub ParseEddystone(beacon1 As Beacon, b() As Byte, offset As Int) As Byte
	Dim hex As String = bc.HexFromBytes(b)
	If b(2 - offset) = 0 Then
		'UID
		beacon1.BeaconType = TypeEddystoneUID
		Dim euid As EddystoneUID
		euid.Namespace = hex.SubString2(8 - offset * 2, 28 - offset * 2)
		euid.Instance = hex.SubString2(28 -offset * 2, 40 - offset * 2)
		beacon1.SpecificData = euid
		beacon1.uniqueid = euid.Namespace & euid.Instance
	Else If b(2 - offset) = 0x10 Then
		'URL
		beacon1.BeaconType = TypeEddystoneURL
		Dim eurl As EddystoneURL
		Dim sb As StringBuilder
		sb.Initialize
		sb.Append(urlSchemes.Get(b(4 - offset)))
		For i = 5 - offset To b.Length - 1
			If b(i) < 20 Then
				sb.Append(urlExpansion.Get(b(i)))
			Else
				sb.Append(Chr(b(i)))
			End If
		Next
		eurl.url = sb.ToString
		beacon1.SpecificData = eurl
		beacon1.uniqueid = eurl.url
	End If
	Return b(3 - offset) - 41
End Sub

Private Sub CompareBytes(b1() As Byte, b2() As Byte, b1Offset As Int) As Boolean 'ignore
	If b1.Length - b1Offset >= b2.Length Then
		For i = 0 To b2.Length - 1
			If b1(i + b1Offset) <> b2(i) Then Return False
		Next
		Return True
	End If
	Return False
End Sub

'based on this answer: http://stackoverflow.com/questions/20416218/understanding-ibeacon-distancing
Private Sub CalculateDistance(tx As Int, rssi As Double) As Double
	If rssi = 0 Then Return -1
	Dim ratio As Double = rssi / tx
	If ratio < 1 Then
		Return Power(ratio, 10)
	Else
		Return 0.89976 * Power(ratio, 7.7095) + 0.111
	End If
End Sub